#---
# Excerpted from "Deploying Rails Applications",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_deploy for more book information.
#---
## caches svn locally to avoid doing a full checkout of your app each time

# written/pulled together by Dreamer3 (Josh Goebel)
# based on http://pastie.caboo.se/1738 by octopod

## HOW TO USE

# require 'config/cache_svn'
#
# set :repository, "svn://your.repository/path/here"
# set :repository_cache, "#{shared_path}/svn_trunk/"

## The code (place in config/cache_svn.rb)


# Expand the Subversion class to support cached repositories
class Capistrano::SCM::Subversion

  def setup_repository_cache(actor)
    command = "#{svn} co -q  #{configuration.repository} #{configuration.repository_cache} &&"
    configuration.logger.debug "Caching SVN repository on remote servers..."
    run_checkout(actor, command, &svn_stream_handler(actor)) 
  end

  def update_repository_cache(actor, revision)
    command = "#{svn} up -r #{revision || 'HEAD'} -q #{configuration.repository_cache} &&"
    run_update(actor, command, &svn_stream_handler(actor)) 
  end

end

# add a tasks related to caching
Capistrano.configuration(:must_exist).load do

  desc <<-DESC
  Setup the cached repository on the server for the first time and
  checkout the latest version there.
  DESC
  task :setup_cached_repository, :roles => [:app, :db, :web] do
    set :revision, "Initial setup checkout" # avoids capistrano trying to find out for us
    run "mkdir -p #{repository_cache}"
    source.setup_repository_cache(self)
  end

  desc <<-DESC
  Update the cached repository and then your app (from the cache) via SVN.
  DESC
  task :update_code, :roles => [:app, :db, :web] do
    source.update_repository_cache(self, revision)

    on_rollback { delete release_path, :recursive => true }

    run "rsync -ax #{repository_cache} #{release_path}/"
    # Remove .svn directories
    run "find #{release_path} -name .svn -print0 | xargs -0 rm -rf"

    run <<-CMD
      rm -rf #{release_path}/log #{release_path}/public/system &&
      ln -nfs #{shared_path}/log #{release_path}/log &&
      ln -nfs #{shared_path}/system #{release_path}/public/system
    CMD

  end

end

